import java.applet.*;
import java.awt.*;
import java.net.*;
public class Main extends Applet implements Runnable {
    Image backBuffer;      // image for double-buffering.
    Graphics backBufferG;  // graphics context for double-buffer.

    AudioClip bgAudio;
    Image image;
    int imageX, imageY;    // current position of image.

    // Applet State Methods
    public void init() {
        backBuffer = createImage(400, 200);
        backBufferG = backBuffer.getGraphics();

        // load resources
        image = getImage(getDocumentBase(), getParameter("image"));
        bgAudio = getAudioClip(getDocumentBase(), getParameter("bg-audio"));

        add(new Button("Click to see source for this applet"));
        resize(400, 200);
    }

    public void start() {
        startTickThread(true);
        bgAudio.loop();
    }

    public void stop() {
        startTickThread(false);
        bgAudio.stop();
    }

    public void destroy() {
        backBufferG.dispose();
        backBuffer.flush();
        image.flush();
    }

    // Paint Methods
    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        int w = size().width;
        int h = size().height;

        if (backBuffer == null 
                || backBuffer.getWidth(this) < w
                || backBuffer.getHeight(this) < h) {
            backBuffer = createImage(w, h);
        }
        backBufferG.clearRect(0, 0, w, h);
        backBufferG.drawImage(image, imageX, imageY, this);
        g.drawImage(backBuffer, 0, 0, this);
    }

    // Event Methods
    public boolean mouseDown(Event evt, int x, int y) {
        if (x > imageX && x < imageX + image.getWidth(this)
            && y > imageY && y < imageY + image.getHeight(this)) {
            startTickThread(false);
            imageX = x-image.getWidth(this)/2;
            imageY = y-image.getHeight(this)/2;
            getAppletContext().showStatus("OWWW! Ya got me! Let go!");
            play(getDocumentBase(), getParameter("audio"));
            repaint();
        }
	return true;
    }

    public boolean mouseDrag(Event evt, int x, int y) {
        if (tickThread == null) {
            imageX = x-image.getWidth(this)/2;
            imageY = y-image.getHeight(this)/2;
            repaint();
        }
        return true;
    }

    public boolean mouseUp(Event evt, int x, int y) {
        if (tickThread == null) {
            startTickThread(true);
            getAppletContext().showStatus("Nah, Nah!");
        }
	return true;
    }

    public boolean action(Event evt, Object arg) {
        if (evt.target instanceof Button) {
            try {
                getAppletContext().showDocument(
                    new URL(getCodeBase(), getParameter("source")));
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
	    return true;
        }
        return false;
    }

    // Tick Thread Methods
    Thread tickThread;
    void startTickThread(boolean start) {
        if (start) {
            tickThread = new Thread(this);
            tickThread.start();
        } else {
            tickThread = null;
        }
    }

    public void run() {
        int delay = Integer.parseInt(getParameter("delay"));
        while (Thread.currentThread() == tickThread) {
            try {
                imageX = (int)Math.floor(Math.random()*size().width);
                imageX = Math.min(imageX, 
                                  size().width-image.getWidth(this));
                imageY = (int)Math.floor(Math.random()*size().height);
                imageY = Math.min(imageY, 
                                  size().height-image.getHeight(this));
                repaint();
                Thread.sleep(delay);
            } catch (InterruptedException e) {
            }
        }
    }

    public String getAppletInfo() {
        return "Patrick Chan and Rosanna Lee (c) 1996";
    }

    String[][] parameterInfo = {
        {"image", "document-based url", "image to move around"},
        {"audio", "document-based url", "sound to play when image is hit"},
        {"bg-audio", "document-based url", "background music"},
        {"audio", "code-based url", "sound to play when image is hit"},
        {"delay", "integer", "delay between moves in milliseconds"},
    };
    public String[][] getParameterInfo() {
        return parameterInfo;
    }
}
