import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
class Main extends Frame {
    TextField textField = new TextField();
    ImageCanvas icv;

    Main(String filename) {
        super("IndexColorModel Example");
        try {
            // Retrieve the image.
            URL url = new URL("file:///" + System.getProperty("user.dir")
               + "/" + filename);
            Image urlImage = getToolkit().getImage(url);

            add("Center", icv = new ImageCanvas(urlImage));
            add("South", textField);
        } catch (Exception e) {
            e.printStackTrace();
        }
        resize(50, 100);
        show();
    }

    public boolean action(Event evt, Object arg) {
        if (evt.target == textField) {
            icv.setAlphaFactor(Double.valueOf(
                textField.getText()).doubleValue());
            return true;
        }
        return false;
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    Image newImage;
    Image image;
    AlphaFilter imgf = new AlphaFilter();

    ImageCanvas(Image image) {
        this.image = image;
        processImage();
    }

    void setAlphaFactor(double afactor) {
        imgf.setAlphaFactor(afactor);
        processImage();
    }

    public void paint(Graphics g) {
        g.drawImage(newImage, 0, 0, this);
    }

    void processImage() {
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, imgf);
        newImage = getToolkit().createImage(ip);
        repaint();
    }
}

class AlphaFilter extends ImageFilter {
    double afactor;
    ColorModel transColorModel;

    void setAlphaFactor(double afactor) {
        this.afactor = afactor;
    }

    public void setColorModel(ColorModel model) {
        if (model instanceof IndexColorModel) {
            IndexColorModel icm = (IndexColorModel)model;
            byte[] reds = new byte[icm.getMapSize()];
            byte[] greens = new byte[icm.getMapSize()];
            byte[] blues = new byte[icm.getMapSize()];
            byte[] alphas = new byte[icm.getMapSize()];
            byte[] cmap = new byte[icm.getMapSize() * 4];

            icm.getReds(reds);
            icm.getGreens(greens);
            icm.getBlues(blues);
            icm.getAlphas(alphas);

            int j = 0;
            for (int i=0; i<icm.getMapSize(); i++) {
                cmap[j++] = reds[i];
                cmap[j++] = greens[i];
                cmap[j++] = blues[i];
                cmap[j++] = (byte)((alphas[i]&0xff)*afactor);
            }
            transColorModel = new IndexColorModel(icm.getPixelSize(), 
                icm.getMapSize(), cmap, 0, true, icm.getTransparentPixel());
        } else {
            transColorModel = model;
        }
        consumer.setColorModel(transColorModel);
    }

    public void setPixels(int x, int y, int w, int h,
          ColorModel model, byte pixels[], int off, int scansize) {
        consumer.setPixels(x, y, w, h, transColorModel, pixels, off, scansize);
    }

    public void setPixels(int x, int y, int w, int h,
          ColorModel model, int pixels[], int off, int scansize) {
        consumer.setPixels(x, y, w, h, transColorModel, pixels, off, scansize);
    }
}

