import java.awt.*;
import java.util.*;
class Main extends Frame {
    Main(String filename) {
        super("keyDown Example");
        Label label = new Label();

	add("South", label);
	add("Center", new ImageCanvas(getToolkit().getImage(filename), label));
        resize(200, 200);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    // mode constants
    final int LEFT = 0;
    final int TOP = 1;
    final int RIGHT = 2;
    final int BOTTOM = 3;
    final int ALL = 4;
    int mode = TOP;

    Rectangle sel;
    Image image;
    Image backBuffer;
    Graphics backBufferG;
    int imageX, imageY, imageW, imageH;
    Label label;
    int scale = 1;

    ImageCanvas(Image image, Label label) {
        MediaTracker tracker = new MediaTracker(this);

        this.label = label;
        tracker.addImage(this.image = image, 0);
        try {
	    tracker.waitForAll();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
	imageW = image.getWidth(null); 
	imageH = image.getHeight(null);
        sel = new Rectangle(imageW/4, imageH/4, imageW/2, imageH/2);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        if (backBuffer == null) {
            backBuffer = createImage(imageW*scale, imageH*scale);
            backBufferG = backBuffer.getGraphics();
        }
        // Draw the image and selection.
        backBufferG.clearRect(0, 0, imageW*scale, imageH*scale);
        backBufferG.setColor(Color.black);
        backBufferG.drawImage(image, 0, 0, imageW*scale, imageH*scale, this);
        backBufferG.drawRect(sel.x, sel.y, sel.width-scale, sel.height-scale);
        backBufferG.setColor(Color.red);

        // Highlight an edge.
        if (mode == LEFT) {
            backBufferG.drawLine(sel.x, sel.y, 
                sel.x, sel.y+sel.height-scale);
        } else if (mode == TOP) { 
            backBufferG.drawLine(sel.x, sel.y, 
                sel.x+sel.width-scale, sel.y);
        } else if (mode == RIGHT) { 
            backBufferG.drawLine(sel.x+sel.width-scale, sel.y, 
                sel.x+sel.width-scale, sel.y+sel.height-scale);
        } else if (mode == BOTTOM) { 
            backBufferG.drawLine(sel.x, sel.y+sel.height-scale, 
                sel.x+sel.width-scale, sel.y+sel.height-scale);
        }

        imageX = (size().width-imageW*scale)/2;
        imageY = (size().height-imageH*scale)/2;
        g.drawImage(backBuffer, imageX, imageY, this);
        label.setText("x="+sel.x+"   y="+sel.y
                      +"   w="+sel.width+"   h="+sel.height);
    }

    public boolean mouseDown(Event evt, int x, int y) {
        x = (x - imageX) - (x - imageX) % scale;
        y = (y - imageY) - (y - imageY) % scale;
        if (evt.controlDown()) {
            sel.width = x - sel.x;
            if (sel.width < 0) {
                sel.x += sel.width;
                sel.width = -sel.width;
            }
            sel.height = y - sel.y;
            if (sel.height < 0) {
                sel.x += sel.height;
                sel.height = -sel.height;
            }
        } else {
            sel.x = x;
            sel.y = y;
        }
        repaint();
        return true;
    }

    public boolean keyDown(Event evt, int key) {
	switch (key) {
          case '+':    // Zoom in
            scale++;
            backBuffer = null;
            prepareImage(image, imageW*scale, imageH*scale, this);
            break;
          case '-':    // Zoom out
            if (scale >= 2) {
                scale--;
                prepareImage(image, imageW*scale, imageH*scale, this);
            }
            break;
          case ' ':
            mode = (mode + 1) % 5;
            break;
	  case Event.UP:
            if (mode == TOP) {
                if (sel.y > 0) {
                    sel.y -= scale;
                    sel.height += scale;
                }
            } else if (mode == BOTTOM) {
                if (sel.height > 2*scale) {
		    sel.height -= scale;
                }
            } else {
                sel.y = Math.max(0, sel.y-scale);
            }
            break;
	  case Event.DOWN:
            if (mode == TOP) {
                if (sel.height > 2*scale) {
                    sel.y += scale;
                    sel.height -= scale;
                }
            } else if (mode == BOTTOM) {
                if (sel.y + sel.height < imageH*scale) {
                    sel.height += scale;
                }
            } else {
                if (sel.y + sel.height < imageH*scale) {
                    sel.y += scale;
                }
            }
            break;
	  case Event.LEFT:
	  case Event.RIGHT:
            return true;
	}
        repaint();
	return false;
    }
}

